@@ -9,7 +9,7 @@ require DynaLoader;
 use vars qw($VERSION @ISA $__import @preload $pid);
 @ISA = qw(DynaLoader);
 sub dl_load_flags { 0x00 }
-$VERSION = '1.58_4';
+$VERSION = '1.62_1';
 $pid = $$;
 bootstrap Prima $VERSION;
 unless ( UNIVERSAL::can('Prima', 'init')) {
@@ -69,6 +69,7 @@ sub run
 {
 	die "Prima was not properly initialized\n" unless $::application;
 	$::application-> go if $::application-> alive;
+	$::application-> destroy if $::application && $::application-> alive;
 	$::application = undef if $::application and not $::application->alive;
 }
 
@@ -345,8 +346,6 @@ L<Prima::Header> - a multi-tabbed header widget
 
 L<Prima::HelpViewer> - the built-in POD file browser
 
-L<Prima::Image::TransparencyControl> - standard dialog for transparent color index selection
-
 L<Prima::ImageViewer> - bitmap viewer
 
 L<Prima::InputLine> - input line widget
@@ -355,7 +354,7 @@ L<Prima::KeySelector> - key combination widget and routines
 
 L<Prima::Menus> - menu widgets
 
-L<Prima::Label> - static text widget 
+L<Prima::Label> - static text widget
 
 L<Prima::Lists> - user-selectable item list widgets
 
@@ -375,11 +374,9 @@ L<Prima::Sliders> - sliding bars, spin buttons and input lines, dial widget etc.
 
 L<Prima::Spinner> - spinner animation
 
-L<Prima::StartupWindow> - a simplistic startup banner window
-
 L<Prima::TextView> - rich text browser widget
 
-L<Prima::Themes> - widget themes manager
+L<Prima::Widgets> - miscellaneous widget classes
 
 =item Standard dialogs
 
@@ -401,13 +398,17 @@ L<Prima::Dialog::PrintDialog> - standard printer setup dialog
 
 =item Drawing helpers
 
+L<Prima::Drawable::Antialias> - plot antialiased shapes
+
 L<Prima::Drawable::CurvedText> - fit text to path
 
 L<Prima::Drawable::Glyphs> - bi-directional text input and complex scripts output
 
 L<Prima::Drawable::Gradient> - gradient fills for primitives
 
-L<Prima::Drawable::Markup> - Allow markup in widgets
+L<Prima::Drawable::Markup> - allow markup in widgets
+
+L<Prima::Drawable::Metafile> - graphic primitive recorder
 
 L<Prima::Drawable::Path> - stroke and fill complex paths
 
@@ -427,9 +428,11 @@ L<Prima::VB::CfgMaint> - maintains visual builder widget palette configuration
 
 =item PostScript printer interface
 
-L<Prima::PS::Drawable> - PostScript interface to C<Prima::Drawable>
+L<Prima::PS::PostScript> - PostScript interface to C<Prima::Drawable>
 
-L<Prima::PS::Printer> - PostScript interface to C<Prima::Printer>
+L<Prima::PS::PDF> - PDF interface to C<Prima::Drawable>
+
+L<Prima::PS::Printer> - PostScript and PDF interfaces to C<Prima::Printer>
 
 =item C interface to the toolkit
 
@@ -453,16 +456,18 @@ L<Prima::IniFile> - support of Windows-like initialization files
 
 L<Prima::IntUtils> - internal functions
 
+L<Prima::StartupWindow> - a simplistic startup banner window
+
 L<Prima::StdBitmap> - shared access to the standard toolkit bitmaps
 
 L<Prima::Stress> - stress test module
 
+L<Prima::Themes> - widget themes manager
+
 L<Prima::Tie> - tie widget properties to scalars or arrays
 
 L<Prima::Utils> - miscellaneous routines
 
-L<Prima::Widgets> - miscellaneous widget classes
-
 =item System-specific modules and documentation
 
 L<Prima::gp-problems> - Graphic subsystem portability issues
@@ -475,6 +480,8 @@ L<Prima::sys::win32::FileDialog> - Windows file system dialogs
 
 L<Prima::sys::XQuartz> - MacOSX/XQuartz facilities
 
+L<Prima::sys::FS> - unicode-aware core file functions
+
 =item Class information
 
 The Prima manual pages often provide information for more than one Prima class.
@@ -490,7 +497,7 @@ viewer C<podview> ( see File/Run/prima-class ).
 Copyright 1997-2003 The Protein Laboratory, University of Copenhagen. All
 rights reserved.
 
-Copyright 2004-2012 Dmitry Karasik. All rights reserved. 
+Copyright 2004-2020 Dmitry Karasik. All rights reserved. 
 
 This program is distributed under the BSD License.
 
