@@ -77,6 +77,7 @@ sub init {
 		(map { $_ => $profile{$_}} keys %hdrProps),
 		%{$profile{headerProfile}},
 	);
+	$self-> {header}-> send_to_back;
 	my $x = $self->{header}->items;
 	$self->{umap} = [ 0 .. $#$x];
 	$self->$_( $profile{$_}) for qw(autoRecalc columns mainColumn);
@@ -161,7 +162,7 @@ sub draw_items
 				$x = 2 - $o if ($ci);
 				my $c = $clrs[ $focused ? 2 : 0];
 				$canvas-> color( $c), $lc = $c if $c != $lc;
-				$canvas->text_out_bidi($node->[0]->[$ci], $x+$txw, $y);
+				$canvas->text_shape_out($node->[0]->[$ci], $x+$txw, $y);
 			}
 		}
 		$xstart += $wx;
@@ -228,7 +229,7 @@ sub reset_scrolls {
 		) if $self-> {vScroll};
 	}
 
-	if ( $self->{scrollTransaction} != 2) { 
+	if ( $self->{scrollTransaction} != 2) {
 		my @sz = $self-> get_active_area( 2);
 		my @widths = @{ $self->{header}->{widths} or [] };
 		my $iw = $#widths * 2;
@@ -326,16 +327,17 @@ sub autoRecalc {($#_)?$_[0]->set_auto_recalc ($_[1]):return $_[0]->{autoRecalc}
 
 =head1 NAME
 
-Prima::DetailedOutline - a multi-column outline viewer with controlling 
+Prima::DetailedOutline - a multi-column outline viewer with controlling
 header widget.
 
 =head1 SYNOPSIS
 
-  use Prima::DetailedOutline;
+  use Prima qw(DetailedOutline Application);
 
-  my $l = $w-> insert( 'Prima::DetailedList', 
+  my $l = Prima::DetailedOutline->new(
         columns => 2,
         headers => [ 'Column 1', 'Column 2' ],
+	size    => [200, 100],
         items => [
               [ ['Item 1, Col 1', 'Item 1, Col 2'], [
                     [ ['Item 1-1, Col 1', 'Item 1-1, Col 2'] ],
@@ -349,6 +351,11 @@ header widget.
         ],
   );
   $l-> sort(1);
+  run Prima;
+
+=for podview <img src="detailedoutline.gif">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/detailedoutline.gif">
 
 =head1 DESCRIPTION
 
