@@ -18,7 +18,6 @@ use Prima::ScrollBar;
 use Prima::StdBitmap;
 use Prima::IntUtils;
 use Prima::Utils;
-use Cwd;
 
 package
     ci;
@@ -962,17 +961,21 @@ sub set_focused_item
 	my $topSet = undef;
 	if ( $foc >= 0) {
 		my $mc   = $self-> {multiColumn};
-		my ( $rows, $cols) = ($mc and not $self->{vertical}) ?
-			($self-> {columns} || 1, $self-> {whole_rows} || 1) :
-			($self-> {rows}    || 1, $self-> {whole_columns} || 1);
-		if ( $foc < $self-> {topItem}) {
-			$topSet = $mc ?
-				$foc - $foc % $rows :
-				$foc;
-		} elsif ( $foc >= $self-> {topItem} + $rows * $cols) {
-			$topSet = $mc ?
-				$foc - $foc % $rows - $rows * ( $cols - 1) :
-				$foc - $rows + 1;
+		if ( $mc ) {
+			my ( $rows, $cols) = ($mc and not $self->{vertical}) ?
+				($self-> {columns} || 1, $self-> {whole_rows} || 1) :
+				($self-> {rows}    || 1, $self-> {whole_columns} || 1);
+			if ( $foc < $self-> {topItem}) {
+				$topSet = $foc - $foc % $rows;
+			} elsif ( $foc >= $self-> {topItem} + $rows * $cols - 1) {
+				$topSet = $foc - $foc % $rows - $rows * ( $cols - 1);
+			}
+		} else {
+			if ( $foc < $self-> {topItem}) {
+				$topSet = $foc;
+			} elsif ( $foc >= $self-> {topItem} + $self->{whole_rows}) {
+				$topSet = $foc - $self->{whole_rows} + 1;
+			}
 		}
 	}
 	$oldFoc = 0 if $oldFoc < 0;
@@ -1259,12 +1262,17 @@ sub set_top_item
 	if ( $self-> { multiColumn}) {
 		$iw += $self-> {drawGrid};
 		if ( $self-> {vertical}) {
-			return $self-> repaint if $dt % $self->{columns};
+			if ($self->{rows} != 0 && abs($dt) % $self->{rows}) {
+				$a[1] += $self->{yedge} if $self->{integralHeight};
+				$self-> scroll( 0, $ih * $dt, clipRect => \@a);
+				return;
+			}
 
 			if ($self->{integralWidth}) {
 				$a[2] -= $self->{xedge};
-			} else {
-				$self-> invalidate_rect($a[2] - $self->{xedge}, $a[1], $a[2], $a[3])
+			} elsif ( !defined $with_focus_shift || $with_focus_shift < 0 ) {
+				# invalid xedge on the right and exposed stripe on the left make clipRect too large
+				$self-> invalidate_rect($a[2] - $self->{xedge}, $a[1], $a[2], $a[3]);
 			}
 			if ( defined $with_focus_shift ) {
 				if ( $with_focus_shift < 0 ) {
@@ -1276,20 +1284,20 @@ sub set_top_item
 					$a[0] += $iw;
 				}
 			}
-			if (( $self-> {rows} != 0) && ( $dt % $self-> {rows} == 0)) {
-				$self-> scroll(
-					-( $dt / $self-> {rows}) * $iw, 0,
-					clipRect => \@a
-				);
-			} else {
-				$self-> scroll( 0, $ih * $dt, clipRect => \@a);
-			}
+			$self-> scroll(
+				-( $dt / $self-> {rows}) * $iw, 0,
+				clipRect => \@a
+			);
 		} else {
-			return $self-> repaint if $dt % $self->{columns};
+			if ($self->{columns} > 0 && abs($dt) % $self->{columns}) {
+				$a[2] -= $self->{xedge} if $self->{integralWidth};
+				$self-> scroll(- $iw * $dt, 0, clipRect => \@a);
+				return;
+			}
 
 			if ($self->{integralHeight}) {
 				$a[1] += $self->{yedge};
-			} else {
+			} elsif ( !defined $with_focus_shift || $with_focus_shift < 0 ) {
 				$self-> invalidate_rect($a[0], $a[1], $a[2], $a[1] + $self->{yedge})
 			}
 			if ( defined $with_focus_shift ) {
@@ -1302,14 +1310,10 @@ sub set_top_item
 					$self-> invalidate_rect($a[0], $a[3], $a[2], $a[3] + $ih);
 				}
 			}
-			if (( $self-> {whole_columns} != 0) && ( $dt % $self-> {whole_columns} == 0)) {
-				$self-> scroll(
-					0, ( $dt / $self-> {whole_columns}) * $ih,
-					clipRect => \@a
-				);
-			} else {
-				$self-> scroll(- $iw * $dt, 0, clipRect => \@a);
-			}
+			$self-> scroll(
+				0, ( $dt / $self-> {columns}) * $ih,
+				clipRect => \@a
+			);
 		}
 	} else {
 		$a[1] += $self-> {yedge}
@@ -1996,7 +2000,7 @@ stores an array of text scalars in a widget. More elaborated storage
 and representation types are not realized, and the programmer is urged
 to use the more abstract classes to derive own mechanisms.
 For example, for a list of items that contain text strings and icons
-see L<Prima::FileDialog/"Prima::DirectoryListBox">.
+see L<Prima::Dialog::FileDialog/"Prima::DirectoryListBox">.
 To organize an item storage, different from C<Prima::ListBox>, it is
 usually enough to overload either the C<Stringify>, C<MeasureItem>,
 and C<DrawItem> events, or their method counterparts: C<get_item_text>,
@@ -2448,6 +2452,6 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::Widget>, L<Prima::ComboBox>, L<Prima::FileDialog>, F<examples/editor.pl>
+L<Prima>, L<Prima::Widget>, L<Prima::ComboBox>, L<Prima::Dialog::FileDialog>, F<examples/editor.pl>
 
 =cut
