@@ -81,6 +81,7 @@ sub init
 	$self-> {paneSize} = [0,0];
 	$self-> {colorMap} = [];
 	$self-> {fontPalette} = [];
+	$self-> {fontPaletteSize} = 0;
 	$self-> {blocks} = [];
 	$self-> {resolution} = [];
 	$self-> {defaultFontSize} = $self-> font-> size;
@@ -92,7 +93,7 @@ sub init
 	$self->{$_} = $profile{$_} for qw(scrollBarClass hScrollBarProfile vScrollBarProfile);
 	for ( qw( autoHScroll autoVScroll colorMap fontPalette
 				hScroll vScroll borderWidth paneWidth paneHeight
-				offset topLine textDirection textRef))
+				offset topLine textDirection text textRef))
 		{ $self-> $_( $profile{ $_}); }
 	return %profile;
 }
@@ -306,6 +307,7 @@ sub fontPalette
 		encoding => '',
 		pitch    => fp::Default,
 	} if scalar @$fm < 1;
+	$self-> {fontPaletteSize} = @$fm;
 	$self-> repaint;
 }
 
@@ -456,14 +458,19 @@ sub paint_selection
 			my ($offset, $length, undef, $text) = @_;
 			my ($vis_start, $vis_end) = (0,0);
 			my $x = $xy[0];
-			my $glyphs = $self-> text_shape($text);
-			my $selection_map = $glyphs-> selection_map( $sx1, $sx2);
+			my $glyphs  = $self-> text_shape($text);
+			my $indexes = $glyphs->indexes;
+			$length = @{ $glyphs-> glyphs };
+			my $selection_map = $glyphs-> selection_map_glyphs( map { $_ - $offset } ($sx1, $sx2));
 			for ( my $i = 0; $i < $length; $i++) {
 				if ( $selection_map->[$i] != $self->{selectionPaintMode} ) {
-					$x += $draw_text->( $glyphs, $x, $vis_start, $vis_end )
-						if $vis_end > $vis_start;
+					if ($vis_end > $vis_start) {
+						my $curr_index = $indexes->[$i-1];
+						$i++, $vis_end++ while $curr_index == $indexes->[$i];
+						$x += $draw_text->( $glyphs, $x, $vis_start, $vis_end );
+					}
 					$vis_start = $vis_end;
-					$self->{selectionPaintMode} = $selection_map->[$offset + $i];
+					$self->{selectionPaintMode} = $selection_map->[$i];
 				}
 				$vis_end++;
 			}
@@ -515,9 +522,11 @@ sub on_paint
 
 	my ( $sx1, $sy1, $sx2, $sy2) = @{$self-> {selection}};
 
+	my @visited;
 	for my $ymap_i ( int( $cy[0] / YMAX) .. int( $cy[1] / YMAX)) {
 		next unless $self-> {ymap}-> [$ymap_i];
 		for my $j ( @{$self-> {ymap}-> [$ymap_i]}) {
+			next if $visited[$j]++;
 			$b = $$bx[$j];
 			my ( $x, $y) = (
 				$aa[0] - $offset + $$b[ tb::BLK_X],
@@ -600,7 +609,11 @@ sub xy2info
 		my ( $minxdist, $bdist, $bdistid) = ( $self-> {paneWidth} * 2, undef, undef);
 		for ( @{$self-> {ymap}-> [ $ymapix]}) {
 			my $z = $$bx[$_];
-			if ( $y >= $$z[ tb::BLK_Y] && $y < $$z[ tb::BLK_Y] + $$z[ tb::BLK_HEIGHT]) {
+			if (
+				$y >= $$z[ tb::BLK_Y] &&
+				$y < $$z[ tb::BLK_Y] + $$z[ tb::BLK_HEIGHT] &&
+				$$z[tb::BLK_TEXT_OFFSET] >= 0
+			) {
 				if (
 					$x >= $$z[ tb::BLK_X] &&
 					$x < $$z[ tb::BLK_X] + $$z[ tb::BLK_WIDTH]
@@ -692,9 +705,9 @@ sub xy2info
 					tw::ReturnFirstLineLength | tw::BreakSingle
 				);
 				my $indexes = $glyphs->indexes;
-				$ofs = $offset + ( $goffs >= $#$indexes ) ? 
-						($length - 1) : # shouldn't happen
-						($indexes->[$goffs] & ~to::RTL);
+				$ofs = $offset + (( $goffs >= $#$indexes ) ? 
+						($indexes-> [-1]) : # shouldn't happen
+						($indexes->[$goffs] & ~to::RTL));
 				$self-> block_walk_abort;
 			} else {
 				$ofs = $offset + $length - 1;
@@ -1063,6 +1076,7 @@ sub selection
 {
 	return @{$_[0]-> {selection}} unless $#_;
 	my ( $self, $sx1, $sy1, $sx2, $sy2) = @_;
+	($sx1, $sy1, $sx2, $sy2) = ( $sx2, $sy2, $sx1, $sy1 ) if $sy1 > $sy2;
 
 	$sy1 = 0 if $sy1 < 0;
 	$sy2 = 0 if $sy2 < 0;
@@ -1082,7 +1096,7 @@ sub selection
 		$y2 = $osy2;
 		if ( $y1 == $y2) {
 			@old = ( $osx1, $osx2 - 1 );
-			@new = (1,0);
+			@new = ();
 		}
 	} else {
 		( $sy1, $sy2, $sx1, $sx2) = ( $sy2, $sy1, $sx2, $sx1) if $sy2 < $sy1;
@@ -1094,7 +1108,7 @@ sub selection
 			$y2 = $sy2;
 			if ( $y1 == $y2) {
 				@new = ( $sx1, $sx2 - 1 );
-				@old = (1,0);
+				@old = ();
 			}
 		} else {
 			if ( $sy1 == $osy1 && $sx1 == $osx1) {
@@ -1109,8 +1123,8 @@ sub selection
 				$y1 = $sy1;
 				$y2 = $osy1;
 				if ( $sy1 == $osy1) {
-					@old = ( $osx1, -1 );
-					@new = ( $sx1,  -1 );
+					@old = ( $osx1, -1);
+					@new = ( $sx1,  -1);
 				}
 			} else {
 				$y1 = ( $sy1 < $osy1) ? $sy1 : $osy1;
@@ -1150,6 +1164,48 @@ sub selection
 		$clipRect[$_+2] = $aa[$_+2] if $clipRect[$_+2] > $aa[$_+2];
 	}
 
+	if ( $y1 == $y2 ) {
+		if ( grep { $_ == -1 } @old, @new ) {
+			my $bx   = $self->{blocks};
+			my $last = (( $y1 < (@{$bx} - 1)) ?
+				$$bx[$y1 + 1]-> [ tb::BLK_TEXT_OFFSET] :
+				length( ${$self-> {text}})
+			) - $$b[ tb::BLK_TEXT_OFFSET];
+			$old[1] = $last if @old && $old[1] == -1;
+			$new[1] = $last if @new && $new[1] == -1;
+		}
+		my @xy = ( $aa[0] - $self->{offset} + $$b[ tb::BLK_X], 0 );
+		my @cr;
+		my $calc = sub {
+			my ( $offset, $glyphs ) = @_;
+			return $offset ? $self->get_text_width($glyphs, 0, 0, $offset) : 0;
+		};
+		$self-> begin_paint_info;
+
+		$self-> block_walk( $b,
+			trace    => tb::TRACE_GEOMETRY | tb::TRACE_REALIZE_PENS | tb::TRACE_TEXT,
+			canvas   => $self,
+			position => \@xy,
+			text     => sub {
+				my ($offset, $length, undef, $text) = @_;
+				my $glyphs  = $self-> text_shape($text);
+				my $oldmap = @old ? $glyphs-> selection_chunks_glyphs( map { $_ - $offset } @old) : [];
+				my $newmap = @new ? $glyphs-> selection_chunks_glyphs( map { $_ - $offset } @new) : [];
+				my $diff   = $glyphs->selection_diff($oldmap, $newmap);
+				return unless @$diff;
+				$cr[0] //= $xy[0] + $calc->($diff->[0], $glyphs);
+				my $end = 0; $end += $_ for @$diff;
+				$cr[1] = $xy[0] + $calc->($end, $glyphs);
+			},
+		);
+		$self-> end_paint_info;
+		if ( @cr && $cr[0] != $cr[1] ) {
+			$cr[1]++;
+			$clipRect[0] = $cr[0] if $clipRect[0] < $cr[0];
+			$clipRect[2] = $cr[1] if $clipRect[2] > $cr[1];
+		}
+	}
+
 	push @invalid_rects, \@clipRect;
 
 	my @cpr = $self-> get_invalid_rect;
@@ -1279,7 +1335,7 @@ Prima::TextView - rich text browser widget
 
  run Prima;
 
-=for podview <img src="textview.gif" cut=1>
+=for podview <img src="textview.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/textview.gif">
 
@@ -1397,6 +1453,9 @@ Also, C<get_selected_text> returns the text within the selection
 the selected text into the clipboard. The latter action is bound to
 C<Ctrl+Insert> key combination.
 
+A block with TEXT_OFFSET set to -1 will be treated as not containing any text,
+and therefore will not be able to get selected.
+
 =head2 Event rectangles
 
 Partly as an option for future development, partly as a hack a
