@@ -110,6 +110,7 @@ sub pop_mark
 package EditorWindow;
 use vars qw(@ISA);
 @ISA = qw(Prima::Window);
+use Prima::sys::FS;
 
 sub profile_default
 {
@@ -160,6 +161,7 @@ sub profile_default
 				),
 				[ '@'.($::application->textDirection ? '*' : '').'rtl' => 
 					'~Right-to-left' => sub{ $_[0]-> {editor}-> textDirection( $_[2] )}],
+				[ '@*ligation' => '~Ligatures' => sub{ $_[0]-> {editor}-> textLigation( $_[2] )}],
 				[ 'Set ~font' => q(setfont)],
 			]]
 		],
@@ -177,7 +179,7 @@ sub init
 	$self-> menu-> utf-> check if $self-> {utf8} = $profile{utf8};
 	if ( defined $fn) {
 		if ( open FILE, '<'.($profile{utf8} ? ':encoding(utf-8)' : ''), $fn) {
-			if ( ! defined read( FILE, $cap, -s $fn)) {
+			if ( ! defined read( FILE, $cap, _s $fn)) {
 				Prima::MsgBox::message("Cannot read file $fn:$!");
 				$fn = undef;
 			}
