@@ -844,6 +844,8 @@ Image_begin_paint( Handle self)
 		inherited end_paint( self);
 		perl_error();
 	}
+	if (ok)
+		apc_gp_set_antialias( self, var->antialias );
 	return ok;
 }
 
@@ -862,6 +864,8 @@ Image_begin_paint_info( Handle self)
 		inherited end_paint_info( self);
 		perl_error();
 	}
+	if (ok)
+		apc_gp_set_antialias( self, var->antialias );
 	return ok;
 }
 
@@ -1679,7 +1683,12 @@ prepare_fill_context(Handle self, Point translate, PImgPaintContext ctx)
 
 	color2pixel( self, my->get_color(self), ctx->color);
 	color2pixel( self, my->get_backColor(self), ctx->backColor);
-	ctx-> rop    = var->extraROP;
+
+	ctx-> rop = var-> extraROP |
+		(( var-> alpha < 255 ) ?
+			ropSrcAlpha | ( var-> alpha << ropSrcAlphaShift ) :
+			0
+		);
 	ctx-> region = var->regionData ? &var->regionData-> data. box : NULL;
 	ctx-> patternOffset = my->get_fillPatternOffset(self);
 	ctx-> patternOffset.x -= translate.x;
@@ -1714,7 +1723,11 @@ prepare_line_context( Handle self, unsigned char * lp, ImgPaintContext * ctx)
 {
 	color2pixel( self, my->get_color(self), ctx->color);
 	color2pixel( self, my->get_backColor(self), ctx->backColor);
-	ctx->rop    = my->get_rop(self);
+	ctx-> rop = var-> extraROP |
+		(( var-> alpha < 255 ) ?
+			ropSrcAlpha | ( var-> alpha << ropSrcAlphaShift ) :
+			0
+		);
 	ctx->region = var->regionData ? &var->regionData-> data. box : NULL;
 	ctx->transparent = my->get_rop2(self) == ropNoOper;
 	ctx->translate = my->get_translate(self);
@@ -1736,84 +1749,6 @@ prepare_line_context( Handle self, unsigned char * lp, ImgPaintContext * ctx)
 	ctx->linePattern = lp;
 }
 
-Bool
-Image_bar( Handle self, int x1, int y1, int x2, int y2)
-{
-	Point t;
-	Bool ok;
-	ImgPaintContext ctx;
-	if (opt_InPaint)
-		return apc_gp_bar( self, x1, y1, x2, y2);
-
-	t = my->get_translate(self);
-	x1 += t.x;
-	y1 += t.y;
-
-	prepare_fill_context(self, t, &ctx);
-	ok = img_bar( self, x1, y1, x2 - x1 + 1, y2 - y1 + 1, &ctx);
-	my-> update_change(self);
-	return ok;
-}
-
-Bool
-Image_bars( Handle self, SV * rects)
-{
-	Point t;
-	ImgPaintContext ctx;
-	int i, count;
-	Bool ok = true, do_free;
-	Rect * p, * r;
-	if (opt_InPaint)
-		return inherited bars( self, rects);
-
-	if (( p = prima_read_array( rects, "Image::bars", 'i', 4, 0, -1, &count, &do_free)) == NULL)
-		return false;
-	t = my->get_translate(self);
-	prepare_fill_context(self, t, &ctx);
-	for ( i = 0, r = p; i < count; i++, r++) {
-		ImgPaintContext ctx2 = ctx;
-		if ( !( ok &= img_bar( self,
-			r->left,
-			r->bottom,
-			r->right - r->left + 1,
-			r->top - r->bottom + 1,
-			&ctx2))) break;
-	}
-	if ( do_free ) free( p);
-	my-> update_change(self);
-	return ok;
-}
-
-Bool
-Image_clear(Handle self, int x1, int y1, int x2, int y2)
-{
-	Point t;
-	Bool ok;
-	ImgPaintContext ctx;
-	if (opt_InPaint)
-		return inherited clear( self, x1, y1, x2, y2);
-	if ( x1 < 0 && y1 < 0 && x2 < 0 && y2 < 0) {
-		x1 = 0;
-		y1 = 0;
-		x2 = var-> w - 1;
-		y2 = var-> h - 1;
-	}
-	t = my->get_translate(self);
-	x1 += t.x;
-	y1 += t.y;
-	color2pixel( self, my->get_backColor(self), ctx.color);
-	ctx.rop = my->get_rop(self);
-	ctx.region = var->regionData ? &var->regionData-> data. box : NULL;
-	memset( ctx.pattern, 0xff, sizeof(ctx.pattern));
-	ctx.patternOffset.x = ctx.patternOffset.y = 0;
-	ctx.patternOffset.x -= t.x;
-	ctx.patternOffset.y -= t.y;
-	ctx.transparent = false;
-	ok = img_bar( self, x1, y1, x2 - x1 + 1, y2 - y1 + 1, &ctx);
-	my-> update_change(self);
-	return ok;
-}
-
 static Bool
 integral_rotate( Handle self, int degrees)
 {
@@ -2013,7 +1948,37 @@ Image_premultiply_alpha( Handle self, SV * alpha)
 		my-> set_type( self, oldType );
 	else
 		my-> update_change( self );
-} 
+}
+
+int
+Image_alpha( Handle self, Bool set, int alpha)
+{
+	if ( is_opt(optInDraw) || is_opt(optInDrawInfo))
+		return inherited alpha(self,set,alpha);
+
+	if (set) {
+		if ( alpha < 0 ) alpha = 0;
+		if ( alpha > 255 ) alpha = 255;
+		if (( alpha < 255 ) && !my->can_draw_alpha(self))
+			alpha = 255;
+		inherited alpha(self,set,alpha);
+	}
+	return var->alpha;
+}
+
+Bool
+Image_antialias( Handle self, Bool set, Bool aa)
+{
+	if ( is_opt(optInDraw) || is_opt(optInDrawInfo))
+		return inherited antialias(self,set,aa);
+
+	if (set) {
+		if ( aa && !my->can_draw_alpha(self))
+			aa = false;
+		var-> antialias = aa;
+	}
+	return var->antialias;
+}
 
 Rect
 Image_clipRect( Handle self, Bool set, Rect r)
@@ -2118,7 +2083,11 @@ primitive( Handle self, Bool fill, char * method, ...)
 	SAVETMPS;
 	strcpy(format, "<");
 	strncat(format, method, 255);
-	ret = call_perl_indirect( self, fill ? "fill_primitive" : "stroke_primitive", format, true, false, args);
+	ret = call_perl_indirect( self, 
+		var->antialias ?
+			(fill ? "fill_imgaa_primitive" : "stroke_imgaa_primitive") :
+			(fill ? "fill_img_primitive"   : "stroke_img_primitive"),
+		format, true, false, args);
 	va_end( args);
 	r = ret ? SvTRUE( ret) : false;
 	FREETMPS;
@@ -2127,32 +2096,209 @@ primitive( Handle self, Bool fill, char * method, ...)
 }
 
 Bool
-Image_arc( Handle self, int x, int y, int dX, int dY, double startAngle, double endAngle)
+Image_arc( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
 {
 	if ( opt_InPaint) return inherited arc(self, x, y, dX, dY, startAngle, endAngle);
-	return primitive( self, 0, "siiiinn", "arc", x, y, dX-1, dY-1, startAngle, endAngle);
+	while ( startAngle > endAngle ) endAngle += 360.0;
+	return primitive( self, 0, "snnnnnn", "arc", x, y, dX-1, dY-1, startAngle, endAngle);
 }
 
 Bool
-Image_chord( Handle self, int x, int y, int dX, int dY, double startAngle, double endAngle)
+Image_bar( Handle self, double x1, double y1, double x2, double y2)
+{
+	Point t;
+	Bool ok;
+	int _x1, _y1, _x2, _y2;
+	ImgPaintContext ctx;
+	if (opt_InPaint)
+		return inherited bar( self, x1, y1, x2, y2);
+	else if ( var-> antialias ) {
+		ok = primitive( self, 1, "snnnn", "rectangle", x1, y1, x2, y2);
+		my-> update_change(self);
+		return ok;
+	}
+
+	_x1 = x1;
+	_x2 = x2;
+	_y1 = y1;
+	_y2 = y2;
+
+	t = my->get_translate(self);
+	_x1 += t.x;
+	_y1 += t.y;
+
+	prepare_fill_context(self, t, &ctx);
+	ok = img_bar( self, _x1, _y1, _x2 - _x1 + 1, _y2 - _y1 + 1, &ctx);
+	my-> update_change(self);
+	return ok;
+}
+
+Bool
+Image_bars( Handle self, SV * rects)
+{
+	Point t;
+	ImgPaintContext ctx;
+	int i, count;
+	Bool ok = true, do_free;
+	Rect *p, *r;
+	if (opt_InPaint)
+		return inherited bars( self, rects);
+	else if ( var-> antialias ) {
+		NRect *p, *r;
+		if (( p = prima_read_array( rects, "Image::bars", 'd', 4, 0, -1, &count, &do_free)) == NULL)
+			return false;
+		for ( i = 0, r = p; i < count; i++, r++) {
+			ok = primitive( self, 1, "snnnn", "rectangle",
+				r->left,
+				r->bottom,
+				r->right - r->left,
+				r->top - r->bottom
+			);
+			if ( !ok ) break;
+		}
+		if ( do_free ) free( p);
+	} else {
+		if (( p = prima_read_array( rects, "Image::bars", 'i', 4, 0, -1, &count, &do_free)) == NULL)
+			return false;
+		t = my->get_translate(self);
+		prepare_fill_context(self, t, &ctx);
+		for ( i = 0, r = p; i < count; i++, r++) {
+			ImgPaintContext ctx2 = ctx;
+			if ( !( ok &= img_bar( self,
+				r->left,
+				r->bottom,
+				r->right - r->left + 1,
+				r->top - r->bottom + 1,
+				&ctx2))) break;
+		}
+		if ( do_free ) free( p);
+	}
+	my-> update_change(self);
+	return ok;
+}
+
+Bool
+Image_chord( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
 {
 	if ( opt_InPaint) return inherited chord(self, x, y, dX, dY, startAngle, endAngle);
-	return primitive( self, 0, "siiiinn", "chord", x, y, dX-1, dY-1, startAngle, endAngle);
+	return primitive( self, 0, "snnnnnn", "chord", x, y, dX-1, dY-1, startAngle, endAngle);
 }
 
 Bool
-Image_ellipse( Handle self, int x, int y,  int dX, int dY)
+Image_clear(Handle self, double x1, double y1, double x2, double y2)
+{
+	Point t;
+	Bool ok;
+	ImgPaintContext ctx;
+	int _x1, _y1, _x2, _y2;
+	Bool full;
+
+	full = x1 < 0 && y1 < 0 && x2 < 0 && y2 < 0;
+	if (opt_InPaint)
+		return inherited clear( self, x1, y1, x2, y2);
+	else if ( !full && var->antialias ) {
+		Bool ok;
+		Color color;
+		FillPattern fp;
+		color = apc_gp_get_color(self);
+		memcpy(&fp, apc_gp_get_fill_pattern(self), sizeof(FillPattern));
+		apc_gp_set_color(self, apc_gp_get_back_color(self));
+		apc_gp_set_fill_pattern(self, fillPatterns[fpSolid]);
+		ok = primitive( self, 1, "snnnn", "rectangle", x1, y1, x2, y2);
+		apc_gp_set_fill_pattern(self, fp);
+		apc_gp_set_color(self, color);
+		return ok;
+	} else {
+		_x1 = x1;
+		_x2 = x2;
+		_y1 = y1;
+		_y2 = y2;
+		if ( _x1 < 0 && _y1 < 0 && _x2 < 0 && _y2 < 0) {
+			_x1 = 0;
+			_y1 = 0;
+			_x2 = var-> w - 1;
+			_y2 = var-> h - 1;
+		}
+		t = my->get_translate(self);
+		_x1 += t.x;
+		_y1 += t.y;
+		color2pixel( self, my->get_backColor(self), ctx.color);
+		*ctx.backColor = *ctx.color;
+		ctx.rop = my->get_rop(self);
+		ctx.region = var->regionData ? &var->regionData-> data. box : NULL;
+		memset( ctx.pattern, 0xff, sizeof(ctx.pattern));
+		ctx.patternOffset.x = ctx.patternOffset.y = 0;
+		ctx.patternOffset.x -= t.x;
+		ctx.patternOffset.y -= t.y;
+		ctx.transparent = false;
+		ok = img_bar( self, _x1, _y1, _x2 - _x1 + 1, _y2 - _y1 + 1, &ctx);
+	}
+	my-> update_change(self);
+	return ok;
+}
+
+Bool
+Image_ellipse( Handle self, double x, double y,  double dX, double dY)
 {
 	if ( opt_InPaint) return inherited ellipse(self, x, y, dX, dY);
-	return primitive( self, 0, "siiii", "ellipse", x, y, dX-1, dY-1);
+	return primitive( self, 0, "snnnn", "ellipse", x, y, dX-1, dY-1);
 }
 
 Bool
-Image_line(Handle self, int x1, int y1, int x2, int y2)
+Image_fill_chord( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
+{
+	if ( opt_InPaint) return inherited fill_chord(self, x, y, dX, dY, startAngle, endAngle);
+	return primitive( self, 1, "snnnnnn", "chord", x, y, dX-1, dY-1, startAngle, endAngle);
+}
+
+Bool
+Image_fill_ellipse( Handle self, double x, double y,  double dX, double dY)
+{
+	if ( opt_InPaint) return inherited fill_ellipse(self, x, y, dX, dY);
+	return primitive( self, 1, "snnnn", "ellipse", x, y, dX-1, dY-1);
+}
+
+Bool
+Image_fillpoly( Handle self, SV * points)
+{
+	if ( opt_InPaint) return inherited fillpoly(self, points);
+	return primitive( self, 1, "sS", "line", points );
+}
+
+Bool
+Image_fill_sector( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
+{
+	if ( opt_InPaint) return inherited fill_sector(self, x, y, dX, dY, startAngle, endAngle);
+	return primitive( self, 1, "snnnnnn", "sector", x, y, dX-1, dY-1, startAngle, endAngle);
+}
+
+Bool
+Image_flood_fill( Handle self, int x, int y, Color color, Bool singleBorder)
+{
+	Point t;
+	Bool ok;
+	ImgPaintContext ctx;
+	ColorPixel px;
+	if (opt_InPaint)
+		return inherited flood_fill(self, x, y, color, singleBorder);
+
+	t = my->get_translate(self);
+	x += t.x;
+	y += t.y;
+
+	prepare_fill_context(self, t, &ctx);
+	color2pixel( self, color, (Byte*)&px);
+	ok = img_flood_fill( self, x, y, px, singleBorder, &ctx); 
+	my-> update_change(self);
+	return ok;
+}
+
+Bool
+Image_line(Handle self, double x1, double y1, double x2, double y2)
 {
 	if ( opt_InPaint) {
 		return inherited line(self, x1, y1, x2, y2);
-	} else if ( my->get_lineWidth(self) == 0) {
+	} else if ( !var->antialias && (int)(my->get_lineWidth(self) + .5) == 0) {
 		ImgPaintContext ctx;
 		unsigned char lp[256];
 		Point poly[2];
@@ -2163,7 +2309,7 @@ Image_line(Handle self, int x1, int y1, int x2, int y2)
 		poly[1].y = y2;
 		return img_polyline(self, 2, poly, &ctx);
 	} else {
-		return primitive( self, 0, "siiii", "line", x1, y1, x2, y2);
+		return primitive( self, 0, "snnnn", "line", x1, y1, x2, y2);
 	}
 }
 
@@ -2172,7 +2318,7 @@ Image_lines( Handle self, SV * points)
 {
 	if ( opt_InPaint) {
 		return inherited lines(self, points);
-	} else if ( my->get_lineWidth(self) == 0) {
+	} else if ( !var->antialias && (int)(my->get_lineWidth(self) + .5) == 0) {
 		Point * lines, *p;
 		int i, count;
 		Bool ok = true, do_free;
@@ -2197,7 +2343,7 @@ Image_polyline( Handle self, SV * points)
 {
 	if ( opt_InPaint) {
 		return inherited polyline(self, points);
-	} else if ( my->get_lineWidth(self) == 0) {
+	} else if ( !var->antialias && (int)(my->get_lineWidth(self) + .5) == 0) {
 		Point * lines;
 		int count;
 		Bool ok, do_free;
@@ -2215,78 +2361,28 @@ Image_polyline( Handle self, SV * points)
 }
 
 Bool
-Image_rectangle(Handle self, int x1, int y1, int x2, int y2)
+Image_rectangle(Handle self, double x1, double y1, double x2, double y2)
 {
 	if ( opt_InPaint) {
 		return inherited rectangle(self, x1, y1, x2, y2);
-	} else if ( my->get_lineWidth(self) == 0) {
+	} else if ( !var->antialias && (int)(my->get_lineWidth(self) + .5) == 0) {
 		ImgPaintContext ctx;
 		unsigned char lp[256];
 		Point r[5] = { {x1,y1}, {x2,y1}, {x2,y2}, {x1,y2}, {x1,y1} };
 		prepare_line_context( self, lp, &ctx);
 		return img_polyline(self, 5, r, &ctx);
 	} else {
-		return primitive( self, 0, "siiii", "rectangle", x1, y1, x2, y2);
+		return primitive( self, 0, "snnnn", "rectangle", x1, y1, x2, y2);
 	}
 }
 
 Bool
-Image_sector( Handle self, int x, int y, int dX, int dY, double startAngle, double endAngle)
+Image_sector( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
 {
 	if ( opt_InPaint) return inherited sector(self, x, y, dX, dY, startAngle, endAngle);
-	return primitive( self, 0, "siiiinn", "sector", x, y, dX-1, dY-1, startAngle, endAngle);
+	return primitive( self, 0, "snnnnnn", "sector", x, y, dX-1, dY-1, startAngle, endAngle);
 }
 
-Bool
-Image_fill_chord( Handle self, int x, int y, int dX, int dY, double startAngle, double endAngle)
-{
-	if ( opt_InPaint) return inherited fill_chord(self, x, y, dX, dY, startAngle, endAngle);
-	return primitive( self, 1, "siiiinn", "chord", x, y, dX-1, dY-1, startAngle, endAngle);
-}
-
-Bool
-Image_fill_ellipse( Handle self, int x, int y,  int dX, int dY)
-{
-	if ( opt_InPaint) return inherited fill_ellipse(self, x, y, dX, dY);
-	return primitive( self, 1, "siiii", "ellipse", x, y, dX-1, dY-1);
-}
-
-Bool
-Image_fillpoly( Handle self, SV * points)
-{
-	if ( opt_InPaint) return inherited fillpoly(self, points);
-	return primitive( self, 1, "sS", "line", points );
-}
-
-Bool
-Image_fill_sector( Handle self, int x, int y, int dX, int dY, double startAngle, double endAngle)
-{
-	if ( opt_InPaint) return inherited fill_sector(self, x, y, dX, dY, startAngle, endAngle);
-	return primitive( self, 1, "siiiinn", "sector", x, y, dX-1, dY-1, startAngle, endAngle);
-}
-
-Bool
-Image_flood_fill( Handle self, int x, int y, Color color, Bool singleBorder)
-{
-	Point t;
-	Bool ok;
-	ImgPaintContext ctx;
-	ColorPixel px;
-	if (opt_InPaint)
-		return inherited flood_fill(self, x, y, color, singleBorder);
-
-	t = my->get_translate(self);
-	x += t.x;
-	y += t.y;
-
-	prepare_fill_context(self, t, &ctx);
-	color2pixel( self, color, (Byte*)&px);
-	ok = img_flood_fill( self, x, y, px, singleBorder, &ctx); 
-	my-> update_change(self);
-	return ok;
-}
-
-
 #ifdef __cplusplus
 }
 #endif
