@@ -347,6 +347,7 @@ XS(Utils_stat_FROMPERL) {
 	StatRec stats;
 	int ok;
 	Bool link = false;
+	U8 gimme;
 
 	if ( items > 2)
 		croak( "invalid usage of Prima::Utils::stat");
@@ -357,7 +358,11 @@ XS(Utils_stat_FROMPERL) {
 	ok = apc_fs_stat( name, prima_is_utf8_sv(ST(0)), link, &stats);
 	SPAGAIN;
 	SP -= items;
-	if ( ok) {
+	gimme = GIMME_V;
+	if ( gimme != G_ARRAY ) {
+		if ( gimme != G_VOID )
+			XPUSHs(newSViv(ok));
+	} else if ( ok) {
 		EXTEND( sp, 11 );
 		PUSHs( sv_2mortal(newSVuv( stats. dev    )));
 		PUSHs( sv_2mortal(newSVuv( stats. ino    )));
