@@ -1,7 +1,6 @@
 use strict;
 use warnings;
 use Getopt::Long;
-use Prima::noARGV;
 use Prima;
 use Prima::HelpViewer;
 use Prima::Application;
@@ -95,7 +94,7 @@ podview - graphical pod viewer
 
 =head2 DESCRIPTION
 
-The small GUI browser for POD format. Given the argument either
+A small GUI browser for POD format. Given the argument either
 as a file path, or perl module name (f.ex. I<File::Copy>), displays
 the documentation found.
 
@@ -110,3 +109,5 @@ L<Prima::HelpViewer> - menu commands explained
 =head1 AUTHORS
 
 Dmitry Karasik E<lt>dmitry@karasik.eu.orgE<gt>
+
+=cut
